%{
/* File lexlex.l
 * The Unknown Scandinavian was here
 * 9 Nov 1990
 *
 * Lexically analyzing BLISS using lex
 * lex.yy.c is #included in parse.y
 */

#include <strings.h>
#include <ctype.h>

static int keyword ();
static void downcase ();

%}

 
digit		[0-9]
letter		[A-Za-z_$]
string		'([^'\n\f]|'')*'
name		{letter}({letter}|{digit})*
delim   "="|":"|","|"/"|"."|"<"|"["|"("|"-"|"*"|"+"|"^"|">"|"]"|")"|";"
others	  \"|"#"|"&"|"?"|"@"|\\|"`"|"{"|"|"|"}"|"~"


%%

{delim}				{
				  return *yytext;
				}

{digit}+			{
				  return T_DIGITS;
				}

{name}				{
				 int key;

				 downcase (yytext);
				 key = keyword (yytext);
				 if (key == -1) 
				   {
				     return T_NAME;
				   }
				 return key;
				}

"%"{name}			{
				 int key;

				 downcase (yytext);
				 key = keyword (yytext);
				 if (key == -1)
				   yyerror ();
				 return key;
				}

"%"				{
				  return '%';
				}

{string}			{
				/* store that string w/o '' */
				  return T_STRING;
				}

![^\n\f]*[\n\f]			/* ignore trailing comment */  ;

"%("([^\)]|")"[^\%])*")%"	/* ignore embedded comment */  ;

{others}			{
				  yyerror ();
				}
%%


struct dict {
  char *d_text;
  int	d_num;
};

static void downcase (cp)
     char *cp;
{
  while (cp && *cp) {
    if (isupper (*cp))
      *cp = tolower (*cp);
    cp++;
  }
}

static struct dict keywords[] = 
{
  { "%allocation", P_ALLOCATION },	/* percent names */
  { "%ascic", P_ASCIC },
  { "%ascid", P_ASCID },
  { "%ascii", P_ASCII },
  { "%asciz", P_ASCIZ },
  { "%assign", P_ASSIGN },
  { "%b", P_B },
  { "%bliss", P_BLISS },
  { "%bliss16", P_BLISS16 },
  { "%bliss32", P_BLISS32 },
  { "%bliss36", P_BLISS36 },
  { "%bpaddr", P_BPADDR },
  { "%bpunit", P_BPUNIT },
  { "%bpval", P_BPVAL },
  { "%c", P_C },
  { "%char", P_CHAR },
  { "%charcount", P_CHARCOUNT },
  { "%chesc", P_CHESC },		/* Is this a keyword? */
  { "%count", P_COUNT },
  { "%ctce", P_CTCE },
  { "%d", P_D },
  { "%decimal", P_DECIMAL },
  { "%declared", P_DECLARED },
  { "%e", P_E },
  { "%else", P_ELSE },
  { "%error", P_ERROR },
  { "%errormacro", P_ERRORMACRO },
  { "%exactstring", P_EXACTSTRING },
  { "%exititeration", P_EXITITERATION },
  { "%exitmacro", P_EXITMACRO },
  { "%expand", P_EXPAND },
  { "%explode", P_EXPLODE },
  { "%fi", P_FI },
  { "%fieldexpand", P_FIELDEXPAND },
  { "%g", P_G },
  { "%h", P_H },
  { "%identical", P_IDENTICAL },
  { "%if", P_IF },
  { "%inform", P_INFORM },
  { "%isstring", P_ISSTRING },
  { "%length", P_LENGTH },
  { "%ltce", P_LTCE },
  { "%message", P_MESSAGE },
  { "%name", P_NAME },
  { "%nbits", P_NBITS },
  { "%nbitsu", P_NBITSU },
  { "%null", P_NULL },
  { "%number", P_NUMBER },
  { "%o", P_O },
  { "%p", P_P },
  { "%print", P_PRINT },
  { "%quote", P_QUOTE },
  { "%quotename", P_QUOTENAME },
  { "%rad50_10", P_RAD50_10 },
  { "%rad50_11", P_RAD50_11 },
  { "%ref", P_REF },
  { "%remaining", P_REMAINING },
  { "%remove", P_REMOVE },
  { "%require", P_REQUIRE },
  { "%sbttl", P_SBTTL },
  { "%sixbit", P_SIXBIT },
  { "%size", P_SIZE },
  { "%string", P_STRING },
  { "%switches", P_SWITCHES },
  { "%the", P_THE },
  { "%title", P_TITLE },
  { "%unquote", P_UNQUOTE },
  { "%upval", P_UPVAL },
  { "%variant", P_VARIANT },
  { "%warn", P_WARN },
  { "%x", P_X },
  { "addressing_mode", K_ADDRESSING_MODE }, /* Normal names */
  { "align", K_ALIGN },
  { "always", K_ALWAYS },
  { "and", K_AND },
  { "begin", K_BEGIN },
  { "bind", K_BIND },
  { "bit", K_BIT },
  { "builtin", K_BUILTIN },
  { "by", K_BY },
  { "byte", K_BYTE },
  { "case", K_CASE },
  { "codecomment", K_CODECOMMENT },
  { "compiletime", K_COMPILETIME },
  { "decr", K_DECR },
  { "decra", K_DECRA },
  { "decru", K_DECRU },
  { "do", K_DO },
  { "else", K_ELSE },
  { "eludom", K_ELUDOM },
  { "enable", K_ENABLE },
  { "end", K_END },
  { "eql", K_EQL },
  { "eqla", K_EQLA },
  { "eqlu", K_EQLU },
  { "eqv", K_EQV },
  { "exitloop", K_EXITLOOP },
  { "external", K_EXTERNAL },
  { "field", K_FIELD },
  { "forward", K_FORWARD },
  { "from", K_FROM },
  { "geq", K_GEQ },
  { "geqa", K_GEQA },
  { "gequ", K_GEQU },
  { "global", K_GLOBAL },
  { "gtr", K_GTR },
  { "gtra", K_GTRA },
  { "gtru", K_GTRU },
  { "if", K_IF },
  { "incr", K_INCR },
  { "incra", K_INCRA },
  { "incru", K_INCRU },
  { "initial", K_INITIAL },
  { "inrange", K_INRANGE },
  { "iopage", K_IOPAGE },
  { "keywordmacro", K_KEYWORDMACRO },
  { "label", K_LABEL },
  { "leave", K_LEAVE },
  { "leq", K_LEQ },
  { "leqa", K_LEQA },
  { "lequ", K_LEQU },
  { "library", K_LIBRARY },
  { "linkage", K_LINKAGE },
  { "literal", K_LITERAL },
  { "local", K_LOCAL },
  { "long", K_LONG },
  { "lss", K_LSS },
  { "lssa", K_LSSA },
  { "lssu", K_LSSU },
  { "macro", K_MACRO },
  { "map", K_MAP },
  { "mod", K_MOD },
  { "module", K_MODULE },
  { "neq", K_NEQ },
  { "neqa", K_NEQA },
  { "nequ", K_NEQU },
  { "not", K_NOT },
  { "novalue", K_NOVALUE },
  { "of", K_OF },
  { "or", K_OR },
  { "otherwise", K_OTHERWISE },
  { "outrange", K_OUTRANGE },
  { "own", K_OWN },
  { "plit", K_PLIT },
  { "preset", K_PRESET },
  { "psect", K_PSECT },
  { "record", K_RECORD },
  { "ref", K_REF },
  { "register", K_REGISTER },
  { "rep", K_REP },
  { "require", K_REQUIRE },
  { "return", K_RETURN },
  { "routine", K_ROUTINE },
  { "select", K_SELECT },
  { "selecta", K_SELECTA },
  { "selectone", K_SELECTONE },
  { "selectonea", K_SELECTONEA },
  { "selectoneu", K_SELECTONEU },
  { "selectu", K_SELECTU },
  { "set", K_SET },
  { "show", K_SHOW },
  { "signed", K_SIGNED },
  { "stacklocal", K_STACKLOCAL },
  { "structure", K_STRUCTURE },
  { "switches", K_SWITCHES },
  { "tes", K_TES },
  { "then", K_THEN },
  { "to", K_TO },
  { "undeclare", K_UNDECLARE },
  { "unsigned", K_UNSIGNED },
  { "until", K_UNTIL },
  { "uplit", K_UPLIT },
  { "volatile", K_VOLATILE },
  { "weak", K_WEAK },
  { "while", K_WHILE },
  { "with", K_WITH },
  { "word", K_WORD },
  { "xor", K_XOR },
  { NULL, -1 }
};

#define NKEYW 177

static int keyword (cp)
     char *cp;				/* It better be lower case */
{
  int first, last, half, c;

  first = 0;				/* inclusive */
  last = NKEYW;				/* exclusive */
  while (first < last)
    {
      half = (first + last) / 2;
      c = strcmp (cp, keywords[half].d_text);
      if (c == 0)
	return keywords[half].d_num;
      else if (c < 0)
	last = half;
      else
	first = half + 1;
    }
  return -1;				/* Not a keyword */
}
