MODULE PSINT( %TITLE'Bliss-36 PS_INTERRUPT Example'
VERSION='1(1)',
	      MAIN=REPLAY )=
BEGIN

!++
!
! FUNCTION
!
!	Example showing the use of the PS_INTERRUPT linkage
!	declaration in Bliss-36.
!
!	This example types a specified file to the TTY
!	in 132 columns. Control-C must be typed in order
!	to exit, causing the terminal width to be reset
!	to its original setting. If a Control-Y is typed
!	the program will prompt for another file.
!
!--

LINKAGE PSI = PS_INTERRUPT;

FORWARD ROUTINE

    REPLAY	: NOVALUE,
    DISPLAY	: NOVALUE,
    ENAPSI	: NOVALUE,	! Sets up and enables the PSI
    CTRCLC	: PSI NOVALUE,	! The Control-C software interrupt
				! handler
    CTRLY	: PSI NOVALUE,	! The Control-C software interrupt
				! handler
    TTYSET	: NOVALUE,	! Sets up terminal characteristics
    FILIO	: NOVALUE,	! Performs file I/O
    TTYRES	: NOVALUE,	! Resets terminal characteristics
    DISPSI	: NOVALUE,	! Disables the PSI
    DSPHDL,			! Condition Handler for the DISPLAY
				! routine
    PSIHDL;			! Condition Handler for the DISPLAY
				! routine

LIBRARY 'BLI:TENDEF';

UNDECLARE

    $CHCRT,			! Declared in both libraries
    $CHLFD;

LIBRARY 'BLI:MONSYM';

REQUIRE 'MJSYS';		! Define needed JSYS calls

LITERAL

    true = 1,
    false = 0,
    ss$cntrl_y = 1,		! Control-Y flag
    intch1 = 1,			! Interrupt channel 1
    intch2 = 2,			! Interrupt channel 2
    bufsiz = 132,		! Size of the file I/O buffer
    typwid = 132;		! Terminal output width

MACRO

    ERROR = ( ERSTR($PRIOU, ($FHSLF^18) OR %O'777777', 0);
		      HALTF(); )%,

    XPDP10_BITS[BITS] = 1^(35-BITS)%,

    PDP10_BITS[] = (0 OR XPDP10_BITS(%REMAINING))%;

OWN
    pclevl,
    levtab : VECTOR[3]
	INITIAL(pclevl, REP 2 OF (0)),

    chntab : VECTOR[36]
	INITIAL(0,(1^18 OR CTRLC),
		  (1^18 OR CTRLY),
		  REP 33 OF (0)),
    filbuf :
	VECTOR[CH$ALLOCATION(bufsiz)],
    filjfn : VOLATILE,
    ttyjfn :
	VOLATILE INITIAL(-1),
    width : VOLATILE;

ROUTINE REPLAY : NOVALUE =
!+
! FUNCTION
!
!	This is the main routine which causes the program
!	to continually loop until a Control-C is typed or
!	an error occurs.
!-

    BEGIN

    WHILE true DO DISPLAY();

    END;

ROUTINE DISPLAY : NOVALUE =
!+
! FUNCTION
!
!	This routine enables a condition handler
!	and calls all the routines which perform
!	the file and terminal I/O.
!-

    BEGIN

    ENABLE DSPHDL;
    ENAPSI();
    TTYSET();
    FILIO();
    TTYRES();
    DISPSI();

    END;

ROUTINE ENAPSI : NOVALUE =
!+
! FUNCTION
!
!	Enables the software interrupt system and sets up
!	the program to trap on Control-C and Control-Y.
!-

    BEGIN

    SIR($FHSLF, ((levtab^18) OR chntab));
    EIR($FHSLF);
    AIC($FHSLF, PDP10_BITS(intch1,intch2));
    ATI($TICCC^18 OR intch1);
    ATI($TICCC^18 OR intch2);

    END;


ROUTINE TTYSET : NOVALUE =
!+
! FUNCTION
!
!	Opens the terminal in Image mode and sets
!	the width to 132, saving the original
!	setting.
!-

    BEGIN

    IF not gtjfn(gj_SHT, CH$PTR(UPLIT(%ASCIZ'TTY:'));
		 ttyjfn)
    THEN
	ERROR;

    IF NOT OPENF(.ttyjfn, ( FLD(7,OF_BSZ) OR FLD(%O'10',
		 OF_MOD) OR OF_WR ))
    THEN
	ERROR;

    MTOPR(.ttyjfn, $MORLW;,,width);

    IF .width NEQ typwid
    THEN
	BEGIN

	MTOPR(.ttyjfn, $MOSLW, typwid);

	SOUT(.ttyjfn, CH$PTR(UPLIT
	    (%ASCII %STRING(%CHAR
	    (%CHESC),'[?31'))), 5, 0);
	END
    END;

ROUTINE FILIO : NOVALUE =
!+
! FUNCTION
!
!	This routine prompts the user for a file name, opens
!	the specified file, and outputs it to the terminal.
!-

    BEGIN

    LITERAL
	max_spec = 219;
    LOCAL
	count,
	cond : MONWORD,
	ttybuf : VECTOR[ CH$ALLOCATION(max_spec) ];

    BEGIN

    PSOUT( CH$PTR(UPLIT(%ASCIZ'FILE:')) );

    IF NOT RDTTY(CH$PTR(ttybuf), max_spec, 0)
    THEN
	ERROR;

    IF NOT GTJFN((GJ_OLD OR GJ_SHT),CH$PTR(ttybuf);
		   filfjn)
    THEN
	ERROR;

    IF NOT OPENF(.filjfn, (FLD(7,OF_BSZ) OR OF_RD))
    THEN
	ERROR;
    END;

    WHILE true DO
	BEGIN

	IF NOT SIN(.filjfn,
	 CH$PTR(filbuf),-bufsiz,0;,,count)
	THEN
	    BEGIN
	    GETER($FHSLF;cond);
	    IF .cond[IOX4]
	    THEN
		BEGIN
		IF .COUNT NEQ 0
		THEN
		    SOUT(.ttyjfn,
		      CH$PTR(filbuf),(bufsiz-.count),0);
		EXITLOOP;
		END;
	    ERROR;
	    END;
	SOUT(.ttyjfn,
	  CH$PTR(filbuf),(bufsiz-.count),0);
	END;
    END;

ROUTINE TTYRES : NOVALUE =
!+
! FUNCTION
!
!	Resets the terminal width back to the original
!	setting and closes and releases the JFN.
!-

    BEGIN

    IF (.width NEQ -1) AND
	 (.width NEQ typwid)
    THEN
	BEGIN

	MTOPR(.ttyjfn,
	 $MOSLW,.width);

	SOUT(.ttyjfn, CH$PTR(UPLIT
	    (%ASCII %STRING(%CHAR
	    (%CHESC),'[?31'))), 5, 0);
	width = 0;
	END;

    IF .ttyjfn GEQ 0
    THEN
	BEGIN

	CLOSF(.ttyjfn);
	ttyjfn = 0;
	END;

    END;

ROUTINE DISPSI : NOVALUE =
!+
! FUNCTION
!
!	Disables the software interrupt system.
!-

    BEGIN

    DTI($TICCY);
    DTI($TICCC);
    DIC($FHSLF, PDP10_BITS(intch1,intch2));
    DIR($FHSLF);

    END;

ROUTINE CTRLC : PSI NOVALUE =
!+
! FUNCTION
!
!	This is the Control-C interrupt handler. The handler
!	calls the routine to reset the width of the terminal
!	and then exits.
!
!	If CONTINUE is typed by the user, the terminal is
!	reset to 132 columns and the program continues from
!	the point where the trap occurred.
!-

    BEGIN

    TTYRES();
    HALTF();
    TTYSET();

    END;

ROUTINE CTRLY : PSI NOVALUE =
!+
! FUNCTION
!
!	This is the Control-Y interrupt handler. This
!	routine signals the PSI condition handler.
!-

    BEGIN

    ENABLE PSIHDL;
    SIGNAL(ss$cntrl_y);

    END;

ROUTINE DSPHDL(sig: REF VECTOR, mech : REF VECTOR, enab : REF
VECTOR) =
!+
! FUNCTION
!
!	Condition handler for the routine Display. This
!	routine terminates all I/O and does an Unwind for
!	the Control-Y signal.
!-

    BEGIN

    IF .sig[1] NEQ ss$cntrl_y
    THEN
	RETURN false;		! Resignal for conditions
				! other than Control-Y.
    TTYRES();
    CLOSF(-1);
    DISPSI();
    SETUNWIND();
    true

    END;

ROUTINE PSIHDL(sig: REF VECTOR, mech : REF VECTOR, enab : REF
VECTOR) =
!+
! FUNCTION
!
!	Condition handler for Control-Y. This routine
!	simply resignals.
!-

    BEGIN

    RETURN false

    END;

END
ELUDOM
